package me.lysne.thesis.levert;

import java.awt.image.BufferedImage;

public class CGRAlgorithm implements DNAToImageTranslator {

    private static final int SIZE = 1 << 9;
    private static final int NUM_BINS = 128;
    private static final int STEP = SIZE / NUM_BINS;

    private int[][] bins = new int[NUM_BINS][NUM_BINS];

    public BufferedImage generateImage(String sequence) {

        final int xA = 0;
        final int yA = 0;
        final int xC = SIZE - 1;
        final int yC = 0;
        final int xT = 0;
        final int yT = SIZE - 1;
        final int xG = SIZE - 1;
        final int yG = SIZE - 1;
        final double center = (SIZE - 1) / 2;

        double x = center;
        double y = center;

        for (int i = 0; i < sequence.length(); i++) {
            char base = sequence.charAt(i);
            double dx;
            double dy;

            switch (base) {
                case 'A':
                    dx = (xA - x) / 2;
                    dy = (yA - y) / 2;
                    break;
                case 'C':
                    dx = (xC - x) / 2;
                    dy = (yC - y) / 2;
                    break;
                case 'T':
                    dx = (xT - x) / 2;
                    dy = (yT - y) / 2;
                    break;
                case 'G':
                    dx = (xG - x) / 2;
                    dy = (yG - y) / 2;
                    break;
                default:
                    continue;
            }
            x += dx;
            y += dy;
            bins[((int) (y / STEP))][((int) (x / STEP))]++;
        }

        return generateMap();
    }

    private BufferedImage generateMap() {

        BufferedImage img = new BufferedImage(SIZE, SIZE, BufferedImage.TYPE_INT_RGB);

        for (int y = 0; y < NUM_BINS; y++) {
            for (int x = 0; x < NUM_BINS; x++) {

                double d = 1.0 - (bins[y][x] * 0.1);

                int rgb = Math.max(Util.doubleToRGB(d), 0);

                for (int y0 = 0; y0 < STEP; y0++) {
                    for (int x0 = 0; x0 < STEP; x0++) {
                        img.setRGB(x* STEP +x0, y* STEP +y0, rgb);
                    }
                }

                bins[y][x] = 0;
            }
        }

        return img;
    }
}
