package me.lysne.thesis.levert;

import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.TimeUnit;

public class ImageBatchGenerator {

    private Class<? extends DNAToImageTranslator> translatorClass;
    private File fastaFile;
    private File destinationDir;

    public ImageBatchGenerator(
            Class<? extends DNAToImageTranslator> translatorClass,
            File fastaFile,
            File destinationDir) {
        this.translatorClass = translatorClass;
        this.fastaFile = fastaFile;
        this.destinationDir = destinationDir;
    }

    public void start(int numWorkers) {

        ArrayBlockingQueue<DNASequence> queue = new ArrayBlockingQueue<DNASequence>(100);
        Thread producerThread = new Thread(new ImageGeneratorProducer(queue, fastaFile));
        Thread[] workerThreads = new Thread[numWorkers];

        try {
            producerThread.start();
            for (int i = 0; i < numWorkers; i++) {
                workerThreads[i] = new Thread(new ImageGeneratorWorker(
                        queue,
                        translatorClass.newInstance(),
                        destinationDir
                ));
                workerThreads[i].start();
            }
            producerThread.join();
            for (int i = 0; i < numWorkers; i++) {
                workerThreads[i].join();
            }
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    class ImageGeneratorProducer implements Runnable {

        private ArrayBlockingQueue<DNASequence> queue;
        private File fastaFile;

        ImageGeneratorProducer(ArrayBlockingQueue<DNASequence> queue, File fastaFile) {
            this.queue = queue;
            this.fastaFile = fastaFile;
        }

        public void run() {

            StringBuilder stringBuilder = new StringBuilder();
            BufferedReader reader = null;

            try {
                reader = new BufferedReader(new FileReader(fastaFile));
                String accessionNumber = null;
                String line;
                while ((line = reader.readLine()) != null) {
                    if (line.charAt(0) == '>') {
                        if (accessionNumber != null) {

                            queue.put(new DNASequence(accessionNumber, stringBuilder.toString()));
                            stringBuilder.setLength(0);
                            stringBuilder.trimToSize();
                        }
                        int index = line.indexOf('|') + 1;
                        accessionNumber = line.substring(index, index + 12);
                    } else {
                        stringBuilder.append(line);
                    }
                }
                queue.put(new DNASequence(accessionNumber, stringBuilder.toString()));
            } catch (IOException e) {
                e.printStackTrace();
            } catch (InterruptedException e) {
                e.printStackTrace();
            } finally {
                try {
                    if (reader != null)
                        reader.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    class ImageGeneratorWorker implements Runnable {

        private ArrayBlockingQueue<DNASequence> queue;
        private DNAToImageTranslator translator;
        private File imgDir;

        ImageGeneratorWorker(
                ArrayBlockingQueue<DNASequence> queue,
                DNAToImageTranslator translator,
                File imgDir) {
            this.queue = queue;
            this.translator = translator;
            this.imgDir = imgDir;
        }

        public void run() {
            try {
                DNASequence sequence;
                while ((sequence = queue.poll(5000, TimeUnit.MILLISECONDS)) != null) {
                    BufferedImage img = translator.generateImage(sequence.getSequence());
                    Util.saveImage(img, imgDir.getAbsolutePath() + "/" + sequence.getAccessionNumber()  + ".png");
                }
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
}
