package me.lysne.thesis.levert;

import net.semanticmetadata.lire.DocumentBuilder;
import net.semanticmetadata.lire.ImageSearchHits;
import net.semanticmetadata.lire.ImageSearcher;
import net.semanticmetadata.lire.imageanalysis.LireFeature;
import net.semanticmetadata.lire.impl.GenericFastImageSearcher;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.IOContext;
import org.apache.lucene.store.RAMDirectory;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;

public class Searcher {

    private File indexDir;
    private Class<? extends LireFeature> descriptor;
    private int maxHits;
    private IndexReader indexReader;
    private ImageSearcher imageSearcher;

    public Searcher(File indexDir, Class<? extends LireFeature> descriptor, int maxHits, boolean indexInRAM) {
        this.indexDir = indexDir;
        this.descriptor = descriptor;
        this.maxHits = maxHits;
        readIndex(indexInRAM);
    }

    private void readIndex(boolean indexInRAM) {
        if (!indexDir.exists() || !indexDir.isDirectory())
            return;

        try {
            Directory directory = FSDirectory.open(indexDir);
            if (indexInRAM) {
                indexReader = DirectoryReader.open(new RAMDirectory(directory, IOContext.READ));
            } else {
                indexReader = DirectoryReader.open(directory);
            }
            imageSearcher = new GenericFastImageSearcher(maxHits, descriptor);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public HashMap<String, Float> search(BufferedImage img) throws IOException {

        if (indexReader == null || imageSearcher == null)
            return null;

        HashMap<String, Float> hits = new HashMap<String, Float>(maxHits);

        ImageSearchHits searchHits = imageSearcher.search(img, indexReader);
        for (int i = 0; i < searchHits.length(); i++) {
            String fileName = searchHits.doc(i).getValues(DocumentBuilder.FIELD_NAME_IDENTIFIER)[0];
            hits.put(fileName, searchHits.score(i));
        }

        return hits;
    }
}
